/* 
** Module   : FillInConfig.java
** Abstract : stores FillIn specific state
**
** Copyright (c) 2005-2011, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 EVL 20050725   @21810 Creation based on general concept.
** 002 SIY 20050831   @22502 Refactoring (see @@22394-22409). 
** 003 SIY 20050907   @22635 Fixed use of @inheritDoc tag in javadoc
** 004 SIY 20050913   @22838 Removed redundant disableAutoZap attribute,
**                           fixed handling of the cursorOffset, 
**                           moved getDataType() to superclass, cleanups.
** 005 SIY 20060209   @24467 Major refactoring.
** 006 SIY 20060324   @25281 Added isAutoReturn().
** 007 SIY 20060404   @25258 Support for the text group.
** 008 SIY 20060428   @25086 Moved some members to ControlTextConfig.
** 009 GES 20060803   @28406 Made externalizable for better performance.
**                           Many code standard cleanups.
** 010 SIY 20060926   @30023 Added getters for the next/previous ID's.
** 011 SIY 20100619          Switching to own implementation of character UI.
** 012 SIY 20100804          Changes in imports, some refactorings.
** 013 SVL 20101006          Added NATIVE attribute.
** 014 SIY 20111101          Moved from CHUI package. Minor 
**                           cleanups/refactorings in preparation to GUI client.
** 015 SIY 20111202          Updated imports.
*/

package com.goldencode.p2j.ui;

import java.io.*;

import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * Contains FillIn specific widget state.
 */
public class FillInConfig
extends ControlTextConfig
{
   /** The Progress CURSOR-OFFSET attribute. */
   int cursorOffset = 0;
   
   /** The Progress AUTO-ZAP attribute. */
   boolean autoZap = true;
   
   /** ID of previous widget in the group. */
   private int prevId = -1;

   /** ID of next widget in the group. */
   private int nextId = -1;

   /** The Progress DEBLANK attribute (not implemented). */
   private boolean deblank = false;

   /** The Progress AUTO-RETURN attribute. */
   private boolean autoReturn = false;
   
   /** Group mode flag. */
   private boolean grouping = false;

   /** The Progress NATIVE attribute. */
   private boolean nativ = false;

   /**
    * Default constructor (only used in de-serialization).
    */
   public FillInConfig()
   {
   }
   
   /**
    * Create an instance from other config. References to other widgets are
    * also restored.
    * 
    * @param   instance
    *          Instance to connect.
    * @param   cfg
    *          Instance to copy data from.
    */
   public FillInConfig(Widget instance, ComponentConfig cfg)
   {
      super(instance, cfg);
   }
   
   /**
    * Create empty configuration with all default values.
    * 
    * @param   type
    *          Type of the new configuration.
    * @param   instance
    *          A component to which this instance will be attached.
    */
   public FillInConfig(int type, Widget instance)
   {
      super(type, instance);
   }
   
   
   /**
    * Set value of the AUTO-RETURN attribute.
    * 
    * @param   autoReturn
    *          New value for the attribute.
    */
   @Override
   public void setAutoReturn(boolean autoReturn)
   {
      this.autoReturn = autoReturn;
   }
   
   /**
    * Get value of the AUTO-RETURN attribute.
    * 
    * @return  Current value for the attribute.
    */
   public boolean isAutoReturn()
   {
      return autoReturn;
   }

   /**
    * Set value of the DISABLE-AUTO-ZAP attribute.
    * 
    * @param   disableAutoZap
    *          New value for the attribute.
    */
   @Override
   public void setDisableAutoZap(boolean disableAutoZap)
   {
      setAutoZap(!disableAutoZap);
   }

   /**
    * Set value of the AUTO-ZAP attribute.
    * 
    * @param   autoZap
    *          New value for the attribute.
    */
   @Override
   public void setAutoZap(boolean autoZap)
   {
      this.autoZap = autoZap;
   }

   /**
    * Get value of the AUTO-ZAP attribute.
    * 
    * @return  Current value for the attribute.
    */
   @Override
   public boolean isAutoZap()
   {
      return autoZap;
   }
   
   /**
    * Returns the 1-based cursor's position in the widget beginning from the
    * left edge.
    * 
    * @return  The 1-based cursor's position in the widget beginning from the
    *          left edge.
    */
   @Override
   public int getCursorOffset()
   {
      if (instance == null)
         return 0;
      
      cursorOffset = ((FillIn) instance).getCursorOffset();
      
      return cursorOffset + 1;
   }

   /**
    * Sets the cursor's position.
    * 
    * @param   offset
    *          The 1-based cursor's position in the widget beginning from the
    *          left edge.
    */
   public void setCursorOffset(int offset)
   {
      ((FillIn) instance).setCursorOffset(offset - 1);
      cursorOffset = ((FillIn) instance).getCursorOffset();
   }

   /**
    * Get value of the DEBLANK attribute.
    * 
    * @return  Current value of the attribute.
    */
   public boolean getDeblank()
   {
      //ISSUE: The application does not use this attribute.
      return deblank;
   }

   /**
    * Set new value of the DEBLANK attribute.
    * 
    * @param   deblank
    *          New value for the attribute.
    */
   public void setDeblank(boolean deblank)
   {
      //TODO: this attribute is not supported yet.
      this.deblank = deblank;
   }

   /**
    * Set new values from the provided config instance.
    * 
    * @param   config
    *          Instance to take values from.
    */
   @Override
   public void applyConfig(ComponentConfig config)
   {
      super.applyConfig(config);

      if (config instanceof FillInConfig)
      {
         FillInConfig other = (FillInConfig) config;

         setAutoZap(other.autoZap);
         setAutoReturn(other.autoReturn);
         setDeblank(other.deblank);
         
         setTextGroupNext(other.nextId);
         setTextGroupPrev(other.prevId);
         setTextGroup(other.grouping);
      }
   }

   /**
    * Set text group flag.
    * 
    * @param   enable
    *          New value for the flag.
    */
   public void setTextGroup(boolean enable)
   {
      this.grouping = enable;
      
      if (instance instanceof FillIn)
         ((FillIn) instance).setTextGroup(enable);
   }

   /**
    * Set ID of the next widget in the text group.
    * 
    * @param   id
    *          Widget ID.
    */
   public void setTextGroupNext(int id)
   {
      this.nextId = id;
   }

   /**
    * Set ID of the previous widget in the text group.
    * 
    * @param   id
    *          Widget ID.
    */
   public void setTextGroupPrev(int id)
   {
      this.prevId = id;
   }
   
   /**
    * Get ID on the next component in TEXT clause group.
    * 
    * @return  ID of the next component.
    */
   public int getTextGroupNext()
   {
      return nextId;
   }
   
   /**
    * Get ID on the previous component in TEXT clause group.
    * 
    * @return  ID of the previous component.
    */
   public int getTextGroupPrev()
   {
      return prevId;
   }

   /**
    * Get value of the NATIVE attribute.
    *
    * @return  Current value for the attribute.
    */
   public boolean isNativ()
   {
      return nativ;
   }

   /**
    * Set value of the NATIVE attribute.
    *
    * @param   nativ
    *          New value for the attribute.
    */
   public void setNativ(boolean nativ)
   {
      this.nativ = nativ;
   }

   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.  The parent's state is
    * read first.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      super.readExternal(in);
      cursorOffset = in.readInt();
      prevId       = in.readInt();
      nextId       = in.readInt();
      autoZap      = in.readBoolean();
      deblank      = in.readBoolean();
      autoReturn   = in.readBoolean();
      grouping     = in.readBoolean();
      nativ        = in.readBoolean();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.  The parent's state is
    * written first.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      super.writeExternal(out);
      out.writeInt(cursorOffset);
      out.writeInt(prevId);
      out.writeInt(nextId);
      out.writeBoolean(autoZap);
      out.writeBoolean(deblank);
      out.writeBoolean(autoReturn);
      out.writeBoolean(grouping);
      out.writeBoolean(nativ);
  }
}
